@@ -19,7 +19,12 @@
 #'# 95 % confidence intervals for coefficients of X variables
 #'confint(fit, level = 0.95, parm = "Xcoef")
 #'}
+#'@aliases confint confint.gllvm
+#'@method confint gllvm
+#'@importFrom stats confint
+#'
 #'@export
+#'@export confint.gllvm
 
 confint.gllvm <- function(object, parm=NULL, level = 0.95, ...) {
   if(is.logical(object$sd)) stop("Standard errors for parameters haven't been calculated, so confidence intervals can not be calculated.");
@@ -33,7 +38,7 @@ confint.gllvm <- function(object, parm=NULL, level = 0.95, ...) {
   quadratic <- object$quadratic
   alfa <- (1 - level) / 2
   if(object$row.eff == "random") object$params$row.params = NULL
-
+  
   if(is.null(parm)){
     if (object$family == "negative.binomial") {
       object$params$phi <- NULL
@@ -45,12 +50,16 @@ confint.gllvm <- function(object, parm=NULL, level = 0.95, ...) {
     }
     
     
-    parm_all <- c("sigma.lv","theta", "LvXcoef","beta0", "Xcoef", "B", "row.params", "sigma", "sigmaB", "inv.phi", "phi", "p","zeta")
+    parm_all <- c("sigma.lv","theta", "LvXcoef","beta0", "Xcoef", "B", "row.params", "sigma", "sigmaB", "sigmaLvXcoef", "inv.phi", "phi", "p","zeta")
+    if(object$randomB!=FALSE){
+      object$params$LvXcoef <- NULL
+    }
     parmincl <- parm_all[parm_all %in% names(object$params)]
     cilow <- unlist(object$params[parmincl]) + qnorm(alfa) * unlist(object$sd[parmincl])
     ciup <- unlist(object$params[parmincl]) + qnorm(1 - alfa) * unlist(object$sd[parmincl])
+    
     M <- cbind(cilow, ciup)
-
+    
     colnames(M) <- c(paste(alfa * 100, "%"), paste((1 - alfa) * 100, "%"))
     rnames <- names(unlist(object$params[parmincl]))
     
@@ -62,7 +71,7 @@ confint.gllvm <- function(object, parm=NULL, level = 0.95, ...) {
         rnames[1:((num.lv.c+num.RR) * p)] <- paste(paste("theta.CLV", nr, sep = ""), nc, sep = ".")
       if(quadratic != FALSE)
         rnames[1:((num.lv.c+num.RR) * p *2)] <- c(paste(paste("theta.CLV", nr, sep = ""), nc, sep = "."), paste(paste("theta.CLV", nr, "^2",
-                                                                                                          sep = ""
+                                                                                                                      sep = ""
         ), nc, sep = "."))
       if(quadratic==FALSE)cal <- cal + (num.lv.c+num.RR) * p
       if(quadratic!=FALSE)cal <- cal + (num.lv.c+num.RR) * p * 2
@@ -86,19 +95,19 @@ confint.gllvm <- function(object, parm=NULL, level = 0.95, ...) {
         rnames[1:((num.lv.c+num.RR) * p)] <- paste(paste("theta.CLV", nr, sep = ""), nc, sep = ".")
       if(quadratic != FALSE)
         rnames[1:((num.lv.c+num.RR) * p *2)] <- c(paste(paste("theta.CLV", nr, sep = ""), nc, sep = "."), paste(paste("theta.CLV", nr, "^2",
-                                                                                                             sep = ""
+                                                                                                                      sep = ""
         ), nc, sep = "."))
       if(quadratic==FALSE)cal <- cal + (num.lv.c+num.RR) * p
       if(quadratic!=FALSE)cal <- cal + (num.lv.c+num.RR) * p * 2
-    
-    
+      
+      
       nr <- rep(1:num.lv, each = p)
       nc <- rep(1:p, num.lv)
       if(quadratic == FALSE)
         rnames[((num.lv.c+num.RR)*p+1):((num.lv.c+num.RR)*p+num.lv * p)] <- paste(paste("theta.LV", nr, sep = ""), nc, sep = ".")
       if(quadratic != FALSE)
         rnames[((num.lv.c+num.RR)*p+1):((num.lv.c+num.RR)*p+num.lv * p)] <- c(paste(paste("theta.LV", nr, sep = ""), nc, sep = "."), paste(paste("theta.LV", nr, "^2",
-                                                                                                          sep = ""
+                                                                                                                                                 sep = ""
         ), nc, sep = "."))
       if(quadratic==FALSE)cal <- cal + num.lv * p
       if(quadratic!=FALSE)cal <- cal + num.lv * p * 2
@@ -113,8 +122,8 @@ confint.gllvm <- function(object, parm=NULL, level = 0.95, ...) {
       }
       cal <- cal+num.lv+num.lv.c
     }
-    if((num.lv.c+num.RR)>0){
-      rnames[-c(1:cal)][1:(ncol(object$lv.X)*(num.lv.c+num.RR))] <- paste(rep(colnames(object$lv.X),2),"LV",rep(1:(num.lv.c+num.RR),each=ncol(object$lv.X)),sep=".")
+    if((num.lv.c+num.RR)>0&object$randomB==FALSE){
+      rnames[-c(1:cal)][1:(ncol(object$lv.X)*(num.lv.c+num.RR))] <- paste(colnames(object$lv.X),"LV",rep(1:(num.lv.c+num.RR),each=ncol(object$lv.X)),sep=".")
       cal<-cal + ncol(object$lv.X)*(num.lv.c+num.RR)
     }
     
@@ -130,7 +139,7 @@ confint.gllvm <- function(object, parm=NULL, level = 0.95, ...) {
       rnames[(cal + 1):(cal + length(nr))] <- nr
       cal <- cal + length(nr)
     }
-
+    
     if (is.null(object$TR) && !is.null(object$X)) {
       cnx <- rep(colnames(object$X.design), each = p)
       rnc <- rep(rownames(object$params$Xcoef), nX)
@@ -153,26 +162,29 @@ confint.gllvm <- function(object, parm=NULL, level = 0.95, ...) {
     if (!is.null(object$randomX)) {
       cal <- cal + length(object$params$sigmaB)
     }
+    if (object$randomB!=FALSE) {
+      cal <- cal + length(object$params$sigmaLvXcoef)
+    }
     if(object$family == "negative.binomial"){
-      s <- dim(M)[1]
-      rnames[(cal + 1):s] <- paste("inv.phi", names(object$params$beta0), sep = ".")
+      s <- length(unique(object$disp.group))
+      rnames[(cal + 1):(s+cal)] <- paste("inv.phi", names(object$params$inv.phi), sep = ".")
     }
     
     if(object$family == "tweedie"){
-      s <- dim(M)[1]
-      rnames[(cal + 1):s] <- paste("Dispersion phi", names(object$params$phi), sep = ".")
+      s <-length(unique(object$disp.group))
+      rnames[(cal + 1):(s+cal)] <- paste("Dispersion phi", names(object$params$phi), sep = ".")
     }
     if(object$family == "ZIP"){
-      s <- dim(M)[1]
-      rnames[(cal + 1):s] <- paste("p", names(object$params$p), sep = ".")
+      s <- length(unique(object$disp.group))
+      rnames[(cal + 1):(s+cal)] <- paste("p", names(object$params$p), sep = ".")
     }
     if(object$family == "gaussian"){
-      s <- dim(M)[1]
-      rnames[(cal + 1):s] <- paste("Standard deviations phi", names(object$params$phi), sep = ".")
+      s <- length(unique(object$disp.group))
+      rnames[(cal + 1):(s+cal)] <- paste("Standard deviations phi", names(object$params$phi), sep = ".")
     }
     if(object$family == "gamma"){
-      s <- dim(M)[1]
-      rnames[(cal + 1):s] <- paste("Shape phi", names(object$params$phi), sep = ".")
+      s <- length(unique(object$disp.group))
+      rnames[(cal + 1):(s+cal)] <- paste("Shape phi", names(object$params$phi), sep = ".")
     }
     rownames(M) <- rnames
   } else {
@@ -215,12 +227,19 @@ confint.gllvm <- function(object, parm=NULL, level = 0.95, ...) {
       
       
     }
-    if("LvXcoef"%in%parm){
+    if("LvXcoef"%in%parm&object$randomB==FALSE){
       names(cilow)[gsub("LvXcoef.*","LvXcoef",names(unlist(object$sd[parm])))%in%"LvXcoef"] <-paste(rep(colnames(object$lv.X),2),"LV",rep(1:(num.lv.c+num.RR),each=ncol(object$lv.X)),sep=".")
       names(ciup)[gsub("LvXcoef.*","LvXcoef",names(unlist(object$sd[parm])))%in%"LvXcoef"] <- paste(rep(colnames(object$lv.X),2),"LV",rep(1:(num.lv.c+num.RR),each=ncol(object$lv.X)),sep=".")
     }
-    if("sigma.lv"%in%parm){
+    if("sigmaLvXcoef"%in%parm&object$randomB!=FALSE){
+      if(object$randomB=="LV"){
+        names(cilow)[gsub("sigmaLvXcoef.*","sigmaLvXcoef",names(unlist(object$sd[parm])))%in%"sigmaLvXcoef"]<-  c(paste("sigmaLvXcoef.LV", 1:(num.RR+num.lv.c), sep=""))
+      }else if(object$randomB=="P"){
+        names(cilow)[gsub("sigmaLvXcoef.*","sigmaLvXcoef",names(unlist(object$sd[parm])))%in%"sigmaLvXcoef"]<-  c(paste("sigmaLvXcoef.", colnames(object$lv.X), sep=""))
+      }
       
+    }
+    if("sigma.lv"%in%parm){
       if(num.lv>0&num.lv.c>0){
         names(cilow)[gsub("sigma.lv.*","sigma.lv",names(unlist(object$sd[parm])))%in%"sigma.lv"]<-  c(paste("sigma.CLV", 1:num.lv.c, sep=""),paste("sigma.LV", 1:num.lv, sep=""))
       }else if(num.lv>0){
@@ -229,11 +248,10 @@ confint.gllvm <- function(object, parm=NULL, level = 0.95, ...) {
         names(cilow)[gsub("sigma.lv.*","sigma.lv",names(unlist(object$sd[parm])))%in%"sigma.lv"]<-  c(paste("sigma.CLV", 1:num.lv.c, sep=""))
       }
     }
-      
     
     M <- cbind(cilow, ciup)
     
-
+    
   }
   return(M)
 }
