@@ -32,17 +32,22 @@ se.gllvm <- function(object, ...){
   method <- object$method
   num.lv <- object$num.lv
   num.lv.c <- object$num.lv.c
+  num.lv.cor <- object$num.lvcor
   num.RR <- object$num.RR
   lv.X <- object$lv.X
   
   quadratic <- object$quadratic
   nlvr <- num.lv + num.lv.c 
   nlvr <- num.lv  #+ (object$row.eff=="random")*1
-  cstrucn = switch(object$cstruc, "diag" = 0, "corAR1" = 1, "corExp" = 2, "corCS" = 3)
+  cstrucn = switch(object$corP$cstruc, "diag" = 0, "corAR1" = 1, "corExp" = 2, "corCS" = 3)
+  corwithin <- object$corP$corwithin
   rstruc = object$rstruc
   family = object$family
   familyn <- objrFinal$env$data$family
+  disp.group <- object$disp.group
   out <- list()
+  
+  # Trait model
   if (!is.null(object$TR)) {
     {
       if((object$method %in% c("VA", "EVA"))){
@@ -62,10 +67,14 @@ se.gllvm <- function(object, ...){
       m <- dim(sdr)[1]; incl <- rep(TRUE,m); incld <- rep(FALSE,m)
       
       # Variational params not included for incl
-      incl[names(objrFinal$par)=="Abb"] <- FALSE;
-      if((num.lv.c+num.RR)==0){incl[names(objrFinal$par)=="b_lv"] <- FALSE}
-      if(quadratic == FALSE){incl[names(objrFinal$par)=="lambda2"]<-FALSE}
 
+      if(quadratic == FALSE){incl[names(objrFinal$par)=="lambda2"]<-FALSE}
+      
+      # Terms that are not included in TMBtrait for constrained ordination
+      incl[names(objrFinal$par)=="Ab_lv"] <- FALSE;
+      incl[names(objrFinal$par)=="sigmab_lv"] <- FALSE;
+      incl[names(objrFinal$par)=="b_lv"] <- FALSE
+      
       incl[names(objrFinal$par)=="lg_Ar"] <- FALSE;
       incl[names(objrFinal$par)=="Au"] <- FALSE;
       incl[names(objrFinal$par)=="u"] <- FALSE; 
@@ -79,6 +88,7 @@ se.gllvm <- function(object, ...){
       if(num.lv>0) {
         incld[names(objrFinal$par)=="Au"] <- TRUE
         incld[names(objrFinal$par)=="u"] <- TRUE
+      } else {
         incl[names(objrFinal$par)=="lambda2"] <- FALSE;
         incl[names(objrFinal$par)=="lambda"] <- FALSE;
       }
@@ -106,8 +116,11 @@ se.gllvm <- function(object, ...){
  
       if(method=="LA" || (num.lv==0 && (object$row.eff!="random" && is.null(object$randomX)))){
         covM <- try(MASS::ginv(sdr[incl,incl]))
+        if(inherits(covM, "try-error")) { stop("Standard errors for parameters could not be calculated, due to singular fit.\n") }
         se <- try(sqrt(diag(abs(covM))))
-        if(num.lv > 0 || object$row.eff == "random" || !is.null(object$randomX)) {
+        
+        trpred<-try({
+          if(num.lv > 0 || object$row.eff == "random" || !is.null(object$randomX)) {
           sd.random <- sdrandom(objrFinal, covM, incl)
           prediction.errors <- list()
           
@@ -123,12 +136,16 @@ se.gllvm <- function(object, ...){
           }
           out$prediction.errors <- prediction.errors
         }
+        }, silent=TRUE)
+        if(inherits(trpred, "try-error")) { cat("Prediction errors for latent variables could not be calculated.\n") }
+        
         out$Hess <- list(Hess.full=sdr, incl=incl, cov.mat.mod=covM)
       } else {
         A.mat <- sdr[incl,incl] # a x a
         D.mat <- sdr[incld,incld] # d x d
         B.mat <- sdr[incl,incld] # a x d
         cov.mat.mod <- try(MASS::ginv(A.mat-B.mat%*%solve(D.mat)%*%t(B.mat)),silent=T) 
+        if(inherits(cov.mat.mod, "try-error")) { stop("Standard errors for parameters could not be calculated, due to singular fit.\n") }
         se <- sqrt(diag(abs(cov.mat.mod)))
         
         incla<-rep(FALSE, length(incl))
@@ -146,13 +163,17 @@ se.gllvm <- function(object, ...){
       } else {
         se.beta0 <- se[1:p]; se <- se[-(1:p)];
       }
+      if(family %in% "betaH"){
+        out$sd$betaH <- se[1:length(object$params$betaH)];  se <- se[-(1:length(object$params$betaH))]
+      }
       se.B <- se[1:length(object$params$B)]; se <- se[-(1:length(object$params$B))];
       if(num.lv > 0) {
         se.sigma.lv <- se[1:num.lv];se<-se[-c(1:num.lv)]
         se.lambdas <- matrix(0,p,num.lv); se.lambdas[lower.tri(se.lambdas, diag=FALSE)] <- se[1:(p * num.lv - sum(0:num.lv))];
         colnames(se.lambdas) <- paste("LV", 1:num.lv, sep="");
         rownames(se.lambdas) <- colnames(out$y)
-        out$sd$theta <- se.lambdas; se <- se[-(1:(p * num.lv - sum(0:num.lv)))];
+        out$sd$theta <- se.lambdas; 
+        if((p * num.lv - sum(0:num.lv))>0) se <- se[-(1:(p * num.lv - sum(0:num.lv)))];
         
         if(quadratic==TRUE){
           se.lambdas2 <- matrix(se[1:(p * num.lv)], p, num.lv, byrow = T)  
@@ -166,29 +187,62 @@ se.gllvm <- function(object, ...){
           out$sd$theta <- cbind(out$sd$theta,se.lambdas2)
         }
         out$sd$sigma.lv  <- se.sigma.lv
-        names(out$sd$sigma.lv) <- colnames(out$params$theta[,1:num.lv])
+        names(out$sd$sigma.lv) <- colnames(object$params$theta[,1:num.lv])
+        if(num.lv>0 & family == "betaH"){
+          se.thetaH <- matrix(0,num.lv,p); 
+          se.thetaH[upper.tri(se.thetaH, diag=TRUE)] <- se[1:(sum(upper.tri(se.thetaH, diag=TRUE)))];
+          se <- se[-( 1:sum(upper.tri(se.thetaH, diag=TRUE)) )];
+          out$sd$se.thetaH <- se.thetaH
+        }
+        
       }
-
+      # For correlated LVs:
+      # if(num.lv.cor>0){
+      #   diag(out$sd$theta) <- c(out$sd$sigma.lv)
+      # }
       out$sd$beta0 <- se.beta0; 
       if(!object$beta0com){ names(out$sd$beta0)  <-  colnames(object$y);}
-      out$sd$B <- se.B; names(out$sd$B) <- colnames(objrFinal$env$data$x)
+      out$sd$B <- se.B; 
+      names(out$sd$B) <- names(object$params$B)
       if(object$row.eff=="fixed") {out$sd$row.params <- se.row.params}
       
       if(family %in% c("negative.binomial")) {
-        se.lphis <- se[1:p];  out$sd$inv.phi <- se.lphis*object$params$inv.phi;
+        se.lphis <- se[1:length(unique(disp.group))];  out$sd$inv.phi <- se.lphis*object$params$inv.phi;
         out$sd$phi <- se.lphis*object$params$phi;
-        names(out$sd$inv.phi) <- names(out$sd$phi) <- colnames(object$y);  se <- se[-(1:p)]
+        if(length(unique(disp.group))==p){
+          names(out$sd$phi) <- colnames(object$y);
+        }else if(!is.null(names(disp.group))){
+          try(names(out$sd$phi) <- unique(names(disp.group)),silent=T)
+        }else{
+          names(out$sd$phi) <- paste("Spp. group", as.integer(unique(disp.group)))
+        }
+        names(out$sd$inv.phi) <-  names(out$sd$phi)
+        se <- se[-(1:length(unique(disp.group)))]
       }
-      if(family %in% c("gaussian","tweedie","gamma", "beta")) {
-        se.lphis <- se[1:p];
+      if(family %in% c("gaussian","tweedie","gamma", "beta", "betaH")) {
+        se.lphis <- se[1:length(unique(disp.group))];
         out$sd$phi <- se.lphis*object$params$phi;
-        names(out$sd$phi) <- colnames(object$y);  se <- se[-(1:p)]
+        if(length(unique(disp.group))==p){
+          names(out$sd$phi) <- colnames(object$y);
+        }else if(!is.null(names(disp.group))){
+          try(names(out$sd$phi) <- unique(names(disp.group)),silent=T)
+        }else{
+          names(out$sd$phi) <- paste("Spp. group", as.integer(unique(disp.group)))
+        }
+        
+        se <- se[-(1:length(unique(disp.group)))]
       }
       if(family %in% c("ZIP")) {
-        lp0 <- objrFinal$par[names(objrFinal$par)=="lg_phi"]
-        se.phis <- se[1:p];
-        out$sd$phi <- se.phis*exp(lp0)/(1+exp(lp0))^2;#
-        names(out$sd$phi) <- colnames(object$y);  se <- se[-(1:p)]
+        se.phis <- se[1:length(unique(disp.group))];
+        out$sd$phi <- se.phis*exp(object$lp0)/(1+exp(object$lp0))^2;#
+        if(length(unique(disp.group))==p){
+          names(out$sd$phi) <- colnames(object$y);
+        }else if(!is.null(names(disp.group))){
+          try(names(out$sd$phi) <- unique(names(disp.group)),silent=T)
+        }else{
+          names(out$sd$phi) <- paste("Spp. group", as.integer(unique(disp.group)))
+        }
+        se <- se[-(1:length(unique(disp.group)))]
       }
       if(!is.null(object$randomX)){
         nr <- ncol(xb)
@@ -204,9 +258,18 @@ se.gllvm <- function(object, ...){
         out$sd$sigma <- se[1:length(object$params$sigma)]*c(object$params$sigma[1],rep(1,length(object$params$sigma)-1)); 
         names(out$sd$sigma) <- "sigma"; 
         se=se[-(1:(length(object$params$sigma)))] 
-        if((rstruc ==2 | (rstruc == 1)) & (cstrucn %in% c(1,3))) {out$sd$rho <- se[1]*(1-object$params$rho^2)^1.5; se = se[-1]}
-        if((rstruc ==2 | (rstruc == 1)) & (cstrucn ==2)) {out$sd$rho <- se[1]*object$params$rho; se = se[-1]}
+        if((rstruc ==2 | (rstruc == 1)) & (cstrucn %in% c(1,3))) {out$sd$rho <- se[1]*(1-object$params$rho^2)^1.5; if(length(object$params$rho)) se = se[-(1:length(object$params$rho))]}
+        if((rstruc ==2 | (rstruc == 1)) & (cstrucn %in% c(2,4))) {out$sd$rho <- se[1]*object$params$rho; if(length(object$params$rho)) se = se[-(1:length(object$params$rho))]}
       }
+      if(num.lv.cor>0 & cstrucn>0){
+        if(length(object$params$rho.lv)>0){
+          if((cstrucn %in% c(1,3))) {out$sd$rho.lv <- se[(1:num.lv.cor)]*(1-object$params$rho.lv^2)^1.5; se = se[-(1:num.lv.cor)]}
+          if((cstrucn %in% c(2,4))) {out$sd$rho.lv <- se[(1:length(object$params$rho.lv))]*object$params$rho.lv; se = se[-(1:length(object$params$rho.lv))]}
+          names(out$sd$rho.lv) <- names(object$params$rho.lv)
+        }
+        # if((cstrucn %in% c(2,4))) {out$sd$scaledc <- se[(1:length(object$params$scaledc))]*object$params$scaledc; se = se[-(1:length(out$sd$scaledc))]}
+      }
+
       if(family %in% c("ordinal")){
         y <- object$y
         K = max(y)-min(y)
@@ -259,11 +322,31 @@ se.gllvm <- function(object, ...){
     incl[names(objrFinal$par)%in%c("Br","sigmaB","sigmaij")] <- FALSE
     
     # Variational params not included for incl
+    incl[names(objrFinal$par)=="Ab_lv"] <- FALSE;
     incl[names(objrFinal$par)=="Abb"]=FALSE;
     incl[names(objrFinal$par)=="lg_Ar"] <- FALSE;
     incl[names(objrFinal$par)=="Au"] <- FALSE;
     incl[names(objrFinal$par)=="u"] <- FALSE;
     
+    #slopes for reduced rank predictors
+    if((num.lv.c+num.RR)==0){
+      incl[names(objrFinal$par)=="sigmab_lv"] <- FALSE
+      incl[names(objrFinal$par)=="b_lv"] <- FALSE
+    }else if((num.lv.c+num.RR)>0&object$randomB==FALSE){
+      incl[names(objrFinal$par)=="b_lv"] <- TRUE
+      incl[names(objrFinal$par)=="sigmab_lv"] <- FALSE
+    }else if((num.lv.c+num.RR>0)&object$randomB!=FALSE){
+      incl[names(objrFinal$par)=="b_lv"] <- FALSE
+      
+      inclr[names(objrFinal$par)=="b_lv"] <- TRUE
+      incld[names(objrFinal$par)=="b_lv"] <- TRUE
+      
+      incld[names(objrFinal$par)=="Ab_lv"] <- TRUE
+      
+      incl[names(objrFinal$par)=="sigmab_lv"] <- TRUE
+    }
+    
+    #loadings for quadratic models
     if(quadratic == FALSE){incl[names(objrFinal$par)=="lambda2"]<-FALSE}
     if(familyn!=7) incl[names(objrFinal$par)=="zeta"] <- FALSE
     if(familyn==0 || familyn==2 || familyn==7 || familyn==8) incl[names(objrFinal$par)=="lg_phi"] <- FALSE
@@ -275,6 +358,7 @@ se.gllvm <- function(object, ...){
     } else {
       if(num.RR==0)incl[names(objrFinal$par)=="lambda"] <- FALSE;
       if(num.RR==0)incl[names(objrFinal$par)=="lambda2"] <- FALSE;
+      incl[names(objrFinal$par)=="sigmaLV"] <- FALSE;
     }
 
     if(object$row.eff=="random") {
@@ -290,10 +374,11 @@ se.gllvm <- function(object, ...){
     
     
     
-    if(method=="LA" || ((num.lv+num.lv.c)==0 && (object$method %in% c("VA", "EVA")) && object$row.eff!="random")){
+    if(method=="LA" || ((num.lv+num.lv.c)==0 && (object$method %in% c("VA", "EVA")) && object$row.eff!="random" && object$randomB==FALSE)){
       covM <- try(MASS::ginv(sdr[incl,incl]))
+      if(inherits(covM, "try-error")) { stop("Standard errors for parameters could not be calculated, due to singular fit.\n") }
       se <- try(sqrt(diag(abs(covM))))
-      
+      trpred<-try({
       if((num.lv+num.lv.c) > 0 || object$row.eff == "random"){
         sd.random <- sdrandom(objrFinal, covM, incl, ignore.u = FALSE)
         prediction.errors <- list()
@@ -311,9 +396,15 @@ se.gllvm <- function(object, ...){
           
           prediction.errors$lvs <- cov.lvs
           #sd.random <- sd.random[-(1:(n*num.lv))]
+          if(object$randomB!=FALSE){
+            prediction.errors$Ab.lv <- sd.random$Ab_lv
+          }
         }
         out$prediction.errors <- prediction.errors
       }
+      }, silent=TRUE)
+      if(inherits(trpred, "try-error")) { cat("Prediction errors for latent variables could not be calculated.\n") }
+      
       out$Hess <- list(Hess.full=sdr, incl=incl, cov.mat.mod=covM)
       
     } else {
@@ -321,32 +412,51 @@ se.gllvm <- function(object, ...){
       A.mat <- sdr[incl, incl] # a x a
       D.mat <- sdr[incld, incld] # d x d
       B.mat <- sdr[incl, incld] # a x d
-      cov.mat.mod <- try(MASS::ginv(A.mat-B.mat%*%solve(D.mat)%*%t(B.mat)),silent=T)
+      
+      cov.mat.mod <- try({MASS::ginv(A.mat-B.mat%*%solve(D.mat)%*%t(B.mat))})
+      if(inherits(cov.mat.mod, "try-error")) { stop("Standard errors for parameters could not be calculated, due to singular fit.\n") }
       se <- sqrt(diag(abs(cov.mat.mod)))
       
       incla<-rep(FALSE, length(incl))
       incla[names(objrFinal$par)=="u"] <- TRUE
       out$Hess <- list(Hess.full=sdr, incla = incla, incl=incl, incld=incld, cov.mat.mod=cov.mat.mod)
-      
+
     }
     
     num.X <- 0; if(!is.null(object$X)) num.X <- dim(object$X.design)[2]
-    if(object$row.eff == "fixed") { se.row.params <- c(0,se[1:(n-1)]); names(se.row.params) <- rownames(object$y); se <- se[-(1:(n-1))] }
-    sebetaM <- matrix(se[1:((num.X+1)*p)],p,num.X+1,byrow=TRUE);  se <- se[-(1:((num.X+1)*p))]
-    if((num.lv.c+num.RR)>0){
-    se.LvXcoef <- matrix(se[1:((num.lv.c+num.RR)*ncol(lv.X))],ncol=num.lv.c+num.RR,nrow=ncol(lv.X))
-    se <- se[-c(1:((num.lv.c+num.RR)*ncol(lv.X)))]
-    colnames(se.LvXcoef) <- paste("CLV",1:(num.lv.c+num.RR),sep="")
-    row.names(se.LvXcoef) <- colnames(lv.X)
-    out$sd$LvXcoef <- se.LvXcoef
+    if(object$row.eff == "fixed") { 
+      se.row.params <- c(0,se[1:(n-1)]); 
+      names(se.row.params) <- rownames(object$y); se <- se[-(1:(n-1))] 
+      }
+    sebetaM <- matrix(se[1:((num.X+1)*p)],p,num.X+1,byrow=TRUE);  
+    se <- se[-(1:((num.X+1)*p))]
+    
+    
+    if(family %in% "betaH"){
+      out$sd$betaH <- matrix(se[1:((num.X+1)*p)],p,num.X+1,byrow=TRUE);  se <- se[-(1:((num.X+1)*p))]
+      rownames(out$sd$betaH) <- rownames(object$params$betaH); 
+      colnames(out$sd$betaH) <- colnames(object$params$betaH)
+    }
+
+    if((num.lv.c+num.RR)>0&object$randomB==FALSE){
+      se.LvXcoef <- matrix(se[1:((num.lv.c+num.RR)*ncol(lv.X))],ncol=num.lv.c+num.RR,nrow=ncol(lv.X))
+      se <- se[-c(1:((num.lv.c+num.RR)*ncol(lv.X)))]
+      colnames(se.LvXcoef) <- paste("CLV",1:(num.lv.c+num.RR),sep="")
+      row.names(se.LvXcoef) <- colnames(lv.X)
+      out$sd$LvXcoef <- se.LvXcoef
+    }
+
+    if((num.lv.c+num.lv)>0){
+      se.sigma.lv <- se[1:(num.lv+num.lv.c)];
+      se<-se[-c(1:(num.lv+num.lv.c))]
     }
     
-    if((num.lv.c+num.lv)>0)se.sigma.lv <- se[1:(num.lv+num.lv.c)];se<-se[-c(1:(num.lv+num.lv.c))]
     if(num.lv > 0&(num.lv.c+num.RR)==0) {
       se.lambdas <- matrix(0,p,num.lv); se.lambdas[lower.tri(se.lambdas, diag=FALSE)] <- se[1:(p * num.lv - sum(0:num.lv))];
       colnames(se.lambdas) <- paste("LV", 1:num.lv, sep="");
-      rownames(se.lambdas) <- colnames(out$y)
-      out$sd$theta <- se.lambdas; se <- se[-(1:(p * num.lv - sum(0:num.lv)))];
+      rownames(se.lambdas) <- colnames(object$y)
+      out$sd$theta <- se.lambdas; 
+      if((p * num.lv - sum(0:num.lv))>0) se <- se[-(1:(p * num.lv - sum(0:num.lv)))];
       
       if(quadratic==TRUE){
         se.lambdas2 <- matrix(se[1:(p * num.lv)], p, num.lv, byrow = T)  
@@ -382,7 +492,7 @@ se.gllvm <- function(object, ...){
       se.lambdas[,1:(num.lv.c+num.RR)][lower.tri(se.lambdas[,1:(num.lv.c+num.RR),drop=F], diag=FALSE)] <- se[1:(p * (num.lv.c+num.RR) - sum(0:(num.lv.c+num.RR)))];
       se <- se[-c(1:(p * (num.lv.c+num.RR) - sum(0:(num.lv.c+num.RR))))];
       se.lambdas[,((num.lv.c+num.RR)+1):ncol(se.lambdas)][lower.tri(se.lambdas[,((num.lv.c+num.RR)+1):ncol(se.lambdas),drop=F], diag=FALSE)] <- se[1:(p * num.lv - sum(0:num.lv))];
-      se <- se[-c(1:(p * num.lv - sum(0:num.lv)))]
+      if((p * num.lv - sum(0:num.lv))>0) se <- se[-c(1:(p * num.lv - sum(0:num.lv)))]
       colnames(se.lambdas) <- c(paste("CLV", 1:(num.lv.c+num.RR), sep=""),paste("LV", 1:num.lv, sep=""));
       rownames(se.lambdas) <- colnames(out$y)
       out$sd$theta <- se.lambdas;
@@ -398,13 +508,24 @@ se.gllvm <- function(object, ...){
         se <- se[-(1:((num.lv.c+num.RR)+num.lv))]
         out$sd$theta <- cbind(out$sd$theta,se.lambdas2)
       }
-      
-    }
-    if((num.lv+num.lv.c)>0){
-      out$sd$sigma.lv  <- se.sigma.lv
-      names(out$sd$sigma.lv) <- colnames(out$params$theta[,1:(num.lv+num.lv.c)])
     }
     
+    if(num.lv>0 & family == "betaH"){
+      se.thetaH <- matrix(0,num.lv,p); 
+      se.thetaH[upper.tri(se.thetaH, diag=TRUE)] <- se[1:(sum(upper.tri(se.thetaH, diag=TRUE)))];
+      se <- se[-( 1:sum(upper.tri(se.thetaH, diag=TRUE)) )];
+      out$sd$se.thetaH <- se.thetaH
+    }
+    
+    if((num.lv+num.lv.c)>0){
+      out$sd$sigma.lv  <- se.sigma.lv
+      names(out$sd$sigma.lv) <- colnames(object$params$theta[,1:(num.lv+num.lv.c)])
+    }
+    
+    # For correlated LVs:
+    # if(num.lv.cor>0){
+    #   diag(out$sd$theta) <- c(out$sd$sigma.lv)
+    # }
     out$sd$beta0 <- sebetaM[,1]; names(out$sd$beta0) <- colnames(object$y);
     if(!is.null(object$X)){
       out$sd$Xcoef <- matrix(sebetaM[,-1],nrow = nrow(sebetaM));
@@ -413,28 +534,67 @@ se.gllvm <- function(object, ...){
     if(object$row.eff=="fixed") {out$sd$row.params <- se.row.params}
     
     if(family %in% c("negative.binomial")) {
-      se.lphis <- se[1:p];  out$sd$inv.phi <- se.lphis*object$params$inv.phi;
+      se.lphis <- se[1:length(unique(disp.group))];  out$sd$inv.phi <- se.lphis*object$params$inv.phi;
       out$sd$phi <- se.lphis*object$params$phi;
-      names(out$sd$phi) <- colnames(object$y);  se <- se[-(1:p)]
+      if(length(unique(disp.group))==p){
+        names(out$sd$phi) <- colnames(object$y);
+      }else if(!is.null(names(disp.group))){
+        try(names(out$sd$phi) <- unique(names(disp.group)),silent=T)
+      }else{
+        names(out$sd$phi) <- paste("Spp. group", as.integer(unique(disp.group)))
+      }
+      names(out$sd$inv.phi) <-  names(out$sd$phi)
+      se <- se[-(1:length(unique(disp.group)))]
     }
-    if(family %in% c("tweedie", "gaussian", "gamma","beta")) {
-      se.lphis <- se[1:p];
+    if(family %in% c("gaussian","tweedie","gamma", "beta", "betaH")) {
+      se.lphis <- se[1:length(unique(disp.group))];
       out$sd$phi <- se.lphis*object$params$phi;
-      names(out$sd$phi) <- colnames(object$y);  se <- se[-(1:p)]
+      if(length(unique(disp.group))==p){
+        names(out$sd$phi) <- colnames(object$y);
+      }else if(!is.null(names(disp.group))){
+        try(names(out$sd$phi) <- unique(names(disp.group)),silent=T)
+      }else{
+        names(out$sd$phi) <- paste("Spp. group", as.integer(unique(disp.group)))
+      }
+      
+      se <- se[-(1:length(unique(disp.group)))]
     }
     if(family %in% c("ZIP")) {
-      lp0 <- objrFinal$par[names(objrFinal$par)=="lg_phi"]
-      se.phis <- se[1:p];
-      out$sd$phi <- se.phis*exp(lp0)/(1+exp(lp0))^2;#
-      names(out$sd$phi) <- colnames(object$y);  se <- se[-(1:p)]
+      se.phis <- se[1:length(unique(disp.group))];
+      out$sd$phi <- se.phis*exp(object$lp0)/(1+exp(object$lp0))^2;#
+      if(length(unique(disp.group))==p){
+        names(out$sd$phi) <- colnames(object$y);
+      }else if(!is.null(names(disp.group))){
+        try(names(out$sd$phi) <- unique(names(disp.group)),silent=T)
+      }else{
+        names(out$sd$phi) <- paste("Spp. group", as.integer(unique(disp.group)))
+      }
+      se <- se[-(1:length(unique(disp.group)))]
+    }
+    if((object$randomB!=FALSE)&(num.lv.c+num.RR)>0){
+      se.lsigmab.lv <-  se[1:length(object$paramssigmaLvXcoef)];
+      se <-  se[-c(1:length(object$params$sigmaLvXcoef))]
+      out$sd$sigmaLvXcoef <- se.lsigmab.lv*object$params$sigmaLvXcoef
+      if(object$randomB=="LV")names(out$sd$sigmaLvXcoef) <- paste("CLV",1:(num.lv.c+num.RR), sep="")
+      if(object$randomB=="P")names(out$sd$sigmaLvXcoef) <- colnames(lv.X)
+      # if(object$randomB=="all")names(out$sd$sigmaLvXcoef) <- paste(paste("CLV",1:(num.lv.c+num.RR),sep=""),rep(colnames(lv.X),each=num.RR+num.lv.c),sep=".")
+      if(object$randomB=="single")names(out$sd$sigmaLvXcoef) <- NULL
     }
     if(object$row.eff=="random") { 
       out$sd$sigma <- se[1:length(object$params$sigma)]*c(object$params$sigma[1],rep(1,length(object$params$sigma)-1)); 
       se=se[-(1:length(out$sd$sigma))] 
       names(out$sd$sigma) <- "sigma" 
-      if((rstruc ==2 | (rstruc == 1)) & (cstrucn %in% c(1,3))) {out$sd$rho <- se[1]*(1-object$params$rho^2)^1.5; se = se[-1]}
-      if((rstruc ==2 | (rstruc == 1)) & (cstrucn ==2)) {out$sd$rho <- se[1]*object$params$rho; se = se[-1]}
+      if((rstruc ==2 | (rstruc == 1)) & (cstrucn %in% c(1,3))) {out$sd$rho <- se[1]*(1-object$params$rho^2)^1.5; if(length(object$params$rho)) se = se[-(1:length(object$params$rho))]}
+      if((rstruc ==2 | (rstruc == 1)) & (cstrucn %in% c(2,4))) {out$sd$rho <- se[1]*object$params$rho; if(length(object$params$rho)) se = se[-(1:length(object$params$rho))]}
     }
+    if(num.lv.cor>0 & cstrucn>0){ 
+      if(length(object$params$rho.lv)>0){
+        if((cstrucn %in% c(1,3))) {out$sd$rho.lv <- se[(1:length(object$params$rho.lv))]*(1-object$params$rho.lv^2)^1.5; se = se[-(1:length(object$params$rho.lv))]}
+        if((cstrucn %in% c(2,4))) {out$sd$rho.lv <- se[(1:length(object$params$rho.lv))]*object$params$rho.lv; se = se[-(1:length(object$params$rho.lv))]}
+        names(out$sd$rho.lv) <- names(object$params$rho.lv)
+      }
+    }
+
     if(family %in% c("ordinal")){
       y <- object$y
       K = max(y)-min(y)
